//
//  EngineSettings.h
//  UsenetEngine
//
//  Created by Ger Teunis on 4/1/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UsenetEngine/Obfuscation.h>

@class Scheduler;
@class DownloadGroupsHandler;


@interface EngineSettings : NSObject {
@private
    
    //APP INFO
    NSNumber *FirstRun;
    NSString *PreviousStartApplicationVersion;
	NSString *PreviousStartHelperVersion;
    
    
    //NETWORK
    NSNumber *SocketReadTimeout;
    NSNumber *SocketReadTimeoutSSL;
    NSNumber *DNSCachePeriod;
    NSNumber *SimultaneousTransfer;
    NSNumber *SimultaniousTransferConnectionsPerNZB;

    
    //POSTPROCESSING
    NSArray *FileCleanupExpressions;
    NSNumber *SmartDownloading;
    NSNumber *SkipSampleFiles;
    NSNumber *PostProcessingStartAutomatic;
    NSNumber *CleanupFiles;
    NSNumber *CleanupEmptyFolders;
	NSNumber *CleanupSampleFolder;
    NSNumber *SmartCheck;
    NSNumber *LowPriorityPostprocessing;
    NSNumber *RecursiveUncompress;

    
    //DISKIO
    NSNumber *WriteCacheSize;
    NSNumber *WriteCacheUpperLimitSize;
    NSNumber *MinimalFreeDiskSpaceSystem;
    NSNumber *MinimalFreeDiskSpace;
    NSNumber *StateFileSaveInterval;
    
    
    //FOLDERS
	NSURL *DownloadFolderURL;
    NSURL *CompletedFolderURL;
    NSURL *NZBWatchFolderURL;
	NSData *DownloadFolderSecurityBookmark;
	NSData *CompletedFolderSecurityBookmark;
	NSData *NZBWatchFolderSecurityBookmark;
    NSString *DownloadFolderMountURL;
    NSString *CompletedFolderMountURL;
    NSString *NZBWatchFolderMountURL;
    NSMutableArray *NZBWatchFolderPrevNZBs;
    NSNumber *NZBWatchFolderEnabled;
    NSNumber *NZBWatchFolderUsePolling;
	NSNumber *MoveToCompletedEnabled;

    
    //UNSANDBOX HISTORY FOLDERS
    NSMutableDictionary *UnsandboxHistoryBookmarks;

    
    //CONFIG
    NSNumber *DeleteNZBAfterAdd;
    NSNumber *PauseNZBAfterAdd;
    NSNumber *PauseNZBWithoutPars;
    NSNumber *PausePasswordProtectedArchives;
    NSNumber *ShutdownInsteadOfSleepWhenDone;
    NSNumber *AlwaysKeepSleepWhenDoneEnabled;
    NSNumber *AlwaysAskDownloadLocation;
    NSNumber *NoFolderForOneFileResult;
    NSNumber *PauseOnTooManyFailedArticles;
    NSNumber *DontReplaceDotToSpaces;
    NSNumber *AutoRemoveCompletedNZBs;
    NSNumber *DontWarnForOldNZBs;
    NSNumber *PauseJunkDownloads;
    NSNumber *PauseDamagedWithoutRepairCRC32;
    NSNumber *SearchResultsAdvancedTitles;
    
    
    //REMOTE
    NSNumber *remoteApplicationControlEnabled;
    NSString *remoteApplicationControlAPIKey;
    NSNumber *webInterfaceEnabled;
    NSNumber *remoteHTTPServerPort;
    NSNumber *remoteHTTPServerSSLEnabled;
    NSNumber *remoteApplicationControlWhitelistEnabled;
    NSString *remoteApplicationControlWhitelistIPs;
    

    //MONTHLY VOLUME
    NSNumber *MonthlyBytesTransferred;
    NSNumber *MonthlyBytesTransferredMonth;
    NSNumber *MonthlyBytesTransferredLastWarningMonth;
    NSNumber *MonthlyBytesTransferredWarningMaxMB;
    NSNumber *MonthlyBytesTransferredWarningEnabled;
    
    
    //SPEED LIMIT
    NSNumber *SpeedLimit;
    NSNumber *SpeedLimitEnabled;
	NSDate *SpeedLimitEndDate;

    
    //RSS
    NSNumber *RSSMaxFeedItemsPerFeed;
    NSNumber *RSSMaxFeedItemsPerFeedKeepMinimalDownloaded;
    NSNumber *RSSInterFeedUpdateWait;
    NSNumber *RSSInterNZBDownloadWait;
    NSNumber *RSSInitialUpdateWait;
    NSNumber *RSSComputerWakeupAdditionalWait;
    NSNumber *RSSComputerWakeupDetectionTime;
    NSNumber *RSSFeedUpdateInterval;
    
    
    //GLOBAL CONNECTION ERRORS
    NSNumber *GlobalConnectErrorTimeout;
    NSNumber *GlobalConnectErrorComputerWakeupDetectionTime;
    
    
    //LICENSE DETAILS
    NSString *LicenseName;
    
    
    //SCHEDULER
    Scheduler *SchedulerSettings;
    
    
    //GROUPS HANDLER
    DownloadGroupsHandler *DownloadGroupsHandler;
    
    
    //SEARCH
    NSMutableArray *SearchCustomSearchEngines;
    NSMutableArray *SearchBlackListedPosters;
}

//APP INFO
@property (nonatomic, retain) NSNumber *FirstRun;
@property (nonatomic, retain) NSString *PreviousStartApplicationVersion;
@property (nonatomic, retain) NSString *PreviousStartHelperVersion;


//NETWORK
@property (nonatomic, retain) NSNumber *SocketReadTimeout;
@property (nonatomic, retain) NSNumber *SocketReadTimeoutSSL;
@property (nonatomic, retain) NSNumber *DNSCachePeriod;
@property (nonatomic, retain) NSNumber *SimultaneousTransfer;
@property (nonatomic, retain) NSNumber *SimultaniousTransferConnectionsPerNZB;


//POSTPROCESSING
@property (nonatomic, retain) NSArray *FileCleanupExpressions;
@property (nonatomic, retain) NSNumber *SmartDownloading;
@property (nonatomic, retain) NSNumber *SkipSampleFiles;
@property (nonatomic, retain) NSNumber *PostProcessingStartAutomatic;
@property (nonatomic, retain) NSNumber *CleanupFiles;
@property (nonatomic, retain) NSNumber *CleanupEmptyFolders;
@property (nonatomic, retain) NSNumber *CleanupSampleFolder;
@property (nonatomic, retain) NSNumber *SmartCheck;
@property (nonatomic, retain) NSNumber *LowPriorityPostprocessing;
@property (nonatomic, retain) NSNumber *MoveToCompletedEnabled;
@property (nonatomic, retain) NSNumber *RecursiveUncompress;


//DISKIO
@property (nonatomic, retain) NSNumber *WriteCacheSize;
@property (nonatomic, retain) NSNumber *WriteCacheUpperLimitSize;
@property (nonatomic, retain) NSNumber *MinimalFreeDiskSpaceSystem;
@property (nonatomic, retain) NSNumber *MinimalFreeDiskSpace;
@property (nonatomic, retain) NSNumber *StateFileSaveInterval;


//FOLDERS
@property (nonatomic, retain) NSURL *DownloadFolderURL;
@property (nonatomic, retain) NSURL *CompletedFolderURL;
@property (nonatomic, retain) NSURL *NZBWatchFolderURL;
@property (nonatomic, retain) NSData *DownloadFolderSecurityBookmark;
@property (nonatomic, retain) NSData *CompletedFolderSecurityBookmark;
@property (nonatomic, retain) NSData *NZBWatchFolderSecurityBookmark;
@property (nonatomic, retain) NSString *DownloadFolderMountURL;
@property (nonatomic, retain) NSString *CompletedFolderMountURL;
@property (nonatomic, retain) NSString *NZBWatchFolderMountURL;
@property (nonatomic, retain) NSNumber *NZBWatchFolderEnabled;
@property (nonatomic, retain) NSNumber *NZBWatchFolderUsePolling;
@property (nonatomic, retain) NSMutableArray *NZBWatchFolderPrevNZBs;

//UNSANDBOX HISTORY FOLDERS
@property (nonatomic, retain) NSMutableDictionary *UnsandboxHistoryBookmarks;


//CONFIG
@property (nonatomic, retain) NSNumber *DeleteNZBAfterAdd;
@property (nonatomic, retain) NSNumber *PauseNZBAfterAdd;
@property (nonatomic, retain) NSNumber *PauseNZBWithoutPars;
@property (nonatomic, retain) NSNumber *PausePasswordProtectedArchives;
@property (nonatomic, retain) NSNumber *ShutdownInsteadOfSleepWhenDone;
@property (nonatomic, retain) NSNumber *AlwaysKeepSleepWhenDoneEnabled;
@property (nonatomic, retain) NSNumber *AlwaysAskDownloadLocation;
@property (nonatomic, retain) NSNumber *NoFolderForOneFileResult;
@property (nonatomic, retain) NSNumber *PauseOnTooManyFailedArticles;
@property (nonatomic, retain) NSNumber *DontReplaceDotToSpaces;
@property (nonatomic, retain) NSNumber *AutoRemoveCompletedNZBs;
@property (nonatomic, retain) NSNumber *DontWarnForOldNZBs;
@property (nonatomic, retain) NSNumber *PauseJunkDownloads;
@property (nonatomic, retain) NSNumber *PauseDamagedWithoutRepairCRC32;
@property (nonatomic, retain) NSNumber *SearchResultsAdvancedTitles;


//REMOTE
@property (nonatomic, retain) NSNumber *remoteApplicationControlEnabled;
@property (nonatomic, retain) NSString *remoteApplicationControlAPIKey;
@property (nonatomic, retain) NSNumber *webInterfaceEnabled;
@property (nonatomic, retain) NSNumber *remoteHTTPServerPort;
@property (nonatomic, retain) NSNumber *remoteHTTPServerSSLEnabled;
@property (nonatomic, retain) NSNumber *remoteApplicationControlWhitelistEnabled;
@property (nonatomic, retain) NSString *remoteApplicationControlWhitelistIPs;


//MONTHLY VOLUME
@property (nonatomic, retain) NSNumber *MonthlyBytesTransferred;
@property (nonatomic, retain) NSNumber *MonthlyBytesTransferredMonth;
@property (nonatomic, retain) NSNumber *MonthlyBytesTransferredLastWarningMonth;
@property (nonatomic, retain) NSNumber *MonthlyBytesTransferredWarningMaxMB;
@property (nonatomic, retain) NSNumber *MonthlyBytesTransferredWarningEnabled;


//SPEED LIMIT
@property (nonatomic, retain) NSNumber *SpeedLimit;
@property (nonatomic, retain) NSNumber *SpeedLimitEnabled;
@property (nonatomic, retain) NSDate *SpeedLimitEndDate;


//RSS
@property (nonatomic, retain) NSNumber *RSSMaxFeedItemsPerFeed;
@property (nonatomic, retain) NSNumber *RSSMaxFeedItemsPerFeedKeepMinimalDownloaded;
@property (nonatomic, retain) NSNumber *RSSInterFeedUpdateWait;
@property (nonatomic, retain) NSNumber *RSSInterNZBDownloadWait;
@property (nonatomic, retain) NSNumber *RSSInitialUpdateWait;
@property (nonatomic, retain) NSNumber *RSSComputerWakeupAdditionalWait;
@property (nonatomic, retain) NSNumber *RSSComputerWakeupDetectionTime;
@property (nonatomic, retain) NSNumber *RSSFeedUpdateInterval;


//GLOBAL CONNECTION ERRORS
@property (nonatomic, retain) NSNumber *GlobalConnectErrorTimeout;
@property (nonatomic, retain) NSNumber *GlobalConnectErrorComputerWakeupDetectionTime;


//LICENSE DETAILS
@property (nonatomic, retain) NSString *LicenseName;


//SCHEDULER
@property (nonatomic, retain) Scheduler *SchedulerSettings;


//GROUPS HANDLER
@property (nonatomic, retain) DownloadGroupsHandler *DownloadGroupsHandler;


//SEARCH
@property (nonatomic, retain) NSMutableArray *SearchCustomSearchEngines;
@property (nonatomic, retain) NSMutableArray *SearchBlackListedPosters;


#pragma mark -
#pragma mark Singleton methods
+ (EngineSettings*)sharedInstance;

#pragma mark -
#pragma mark Rest
-(void)setDefaults;
-(void)savePreferences;
-(void)loadPreferences;

@end
