//
//   ConnectionPool.h
//  UsenetEngine
//
//  Created by Ger Teunis on 12/30/09.
//  Copyright 2010 Ger Teunis. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <IOKit/pwr_mgt/IOPMLib.h>
#import "UINotificationsProtocol.h"
#import "NNTPEnums.h"

@class NZB;
@class GrowlMessages;
@class DataWriteThread;
@class PostProcessingThread;
@class NZBParseQueue;
@class DNSCache;
@class ServerHandler;
@class PersistenceHandler;
@class RSSFeedController;
@class EngineSettings;
@class UsenetSearchController;
@class SecurityBookmarkHandler;
@class NetworkAPIDispatcher;

@interface NZBqueue : NSObject {
    NSMutableArray *serverHandlers;
	int realTransferSpeed;
    NSMutableArray *_transferSpeedHistory;
    NSDate *prevTransferSpeedDate;
    NSThread *processDownloadSpeedThread;
	float throttleWait;
	NSLock *nzbQueueLock;
	NSMutableArray *nzbQueue;
	int downloadSpreadIndex;
	bool autoShutdownWhenFinished;
	bool prevQueueAllDone;
    int prevPausedNonErrorNZBs;
	DataWriteThread *dataWriteThread;
	PostProcessingThread *postProcessingThread;
	NZBParseQueue *nzbParseQueue;
    RSSFeedController *rssFeedController;
	bool memoryFullStopDownloading;
	NSLock *upgradePrefsLock;
    NSLock *saveStateFileLock;
    DNSCache *dnsCache;
    PersistenceHandler *persistanceHandler;
    NSLock *rssControllerLoadLock;    
    unsigned long long monthlyBytesTransferred;
    int monthlyBytesTransferredMonth;
    int monthlyBytesTransferredLastWarningMonth;
    bool previousPreventSystemSleep;
    NSObject<UINotificationsProtocol> *uiNotifications;
    EngineSettings *usenetEngineSettings;
	UsenetSearchController *usenetSearchController;
    NSMutableArray *finishedMergingUUIDs;
    bool allPrimaryServersFailing;
    unsigned long long WriteCacheUpperLimitSize;
	bool HelperAppIsAvailable;
    int checkHelperApplicationRetry;
    bool prevSchedulerEnabled;
    SecurityBookmarkHandler *securityBookmarkHandler;
    NetworkAPIDispatcher *networkAPIDispatcher;
}

@property (nonatomic, retain) NSMutableArray *serverHandlers;
@property (nonatomic, retain) NZBParseQueue *nzbParseQueue;
@property (nonatomic, retain) RSSFeedController *rssFeedController;
@property (nonatomic, retain) PostProcessingThread *postProcessingThread;
@property (nonatomic, retain) DataWriteThread *dataWriteThread;
@property (nonatomic) int realTransferSpeed;
@property (nonatomic) float throttleWait;
@property (nonatomic, retain) NSLock *nzbQueueLock;
@property (nonatomic) int downloadSpreadIndex;
@property (nonatomic) bool autoShutdownWhenFinished;
@property (nonatomic) bool prevQueueAllDone;
@property (nonatomic) bool memoryFullStopDownloading;
@property (nonatomic, retain) NSLock *upgradePrefsLock;
@property (nonatomic, retain) DNSCache *dnsCache;
@property (nonatomic, retain) PersistenceHandler *persistanceHandler;
@property (nonatomic, retain) NSLock* rssControllerLoadLock;
@property (nonatomic) unsigned long long monthlyBytesTransferred;
@property (nonatomic) int monthlyBytesTransferredMonth;
@property (nonatomic) int monthlyBytesTransferredLastWarningMonth;
@property (nonatomic, assign) bool previousPreventSystemSleep;
@property (nonatomic, retain) NSObject<UINotificationsProtocol> *uiNotifications;
@property (nonatomic, retain) EngineSettings *usenetEngineSettings;
@property (nonatomic, retain) UsenetSearchController *usenetSearchController;
@property (nonatomic) bool allPrimaryServersFailing;
@property (nonatomic) bool HelperAppIsAvailable;
@property (nonatomic, retain) SecurityBookmarkHandler *securityBookmarkHandler;

#pragma mark -
#pragma mark Singleton methods
+(NZBqueue*)createInstance:(NSObject<UINotificationsProtocol>*)uiNotificationHandler;
+(NZBqueue*)sharedInstance;
+(void)tearDownUsenetEngine;

#pragma mark -
#pragma mark Queue methods
-(void)addNZB:(NZB*)nzb;
-(void)removeNZB:(NZB*)nzb;
-(void)moveUpNZB:(NZB*)nzb;
-(void)moveDownNZB:(NZB*)nzb;
-(void)moveNZBOnTop:(NZB*)nzb;
-(void)moveNZBOnBottom:(NZB*)nzb;
-(void)moveNZB:(NZB*)source beforeNZB:(NZB*)target;
-(void)moveNZB:(NZB*)source afterNZB:(NZB*)target;
-(NSMutableArray*)findDownloadableNZBs:(bool)backupServer;
-(void)sortNZBQueueByState;
-(NSMutableArray*)getNzbQueue;
-(void)sortQueue:(QueueSortType)sortType;
-(bool)nzbIsAlmostReachingMaxRetentions:(NSDate*)nzbAge MaxRetention:(NSNumber**)maxRetention;
-(NSNumber*)maxPrimaryServerRetention;
-(void)addPar2FilesFromNZBTo:(NZB*)addToNZB fromNZBFile:(NSString*)nzbFile;

-(bool)enqueueNZB:(NSString*)nzbFileName 
DestinationFolder:(NSString*)destination 
    OnlyPar2Files:(bool)onlyPars;

-(bool)enqueueNZB:(NSString*)nzbFileName
   desiredUITitle:(NSString*)uiTitle
DestinationFolder:(NSString*)destination
 DeleteAfterParse:(bool)deleteAfterParse
        AddPaused:(bool)addPaused
    OnlyPar2Files:(bool)onlyPars
    DownloadGroup:(NSString*)groupUUID
     MergeAddUUID:(NSString *)mergeAddUUID
     downloadNext:(bool)downloadNext
addedViaWatchName:(NSString*)addedViaRSSWatchName;

-(bool)addNZBFromURL:(NSString *)url
          httpMethod:(NSString*)httpMethod
     httpRequestBody:(NSData*)requestBody
  httpRequestHeaders:(NSDictionary*)requestHeaders
      desiredUITitle:(NSString*)uiTitle
      downloadFolder:(NSString*)downloadFolder
      deleteAfterAdd:(int)deleteAfterAdd
        downloadNext:(int)downloadNext
        mergeAddUUID:(NSString *)mergeAddUUID
   downloadGroupUUID:(NSString *)downloadGroupUUID
            onlyPars:(int)onlyPars
           addPaused:(int)addPaused
   addedViaWatchName:(NSString*)addedViaRSSWatchName;

-(void)resetWatchFolderWatcher;
-(void)addFinishedMergedUUID:(NSString*)mergeUUID;
-(void)finishUpMergedUUIDs;

#pragma mark -
#pragma mark Transfer methods
-(int)averageTransferSpeed;
-(NSMutableArray*)totalTransferSpeedHistory;
-(void)addTransferSpeedToHistory:(int)speed;
-(void)cancelAllTransfers:(id)sender andDeleteFiles:(bool)deleteFiles;
-(void)pauseAllTransfers:(id)sender Reason:(NSString*)pauseReason;
-(void)removeAllFinishedTransfers:(id)sender;
-(IBAction)resumeAllTransfers:(id)sender;

#pragma mark -
#pragma mark Sandbox wrap functions
-(void)unsandboxSoundFiles;

#pragma mark -
#pragma mark Instance methods
-(void)pauseTooDamagedNZBs;
-(void)updateAllPrimaryServersFailing;
-(NSMutableArray*)loadServerHandlersInArray;
-(void)loadServerHandlers;
-(void)saveServerHandlers;
-(bool)processedAllPrimaryServers:(NSMutableArray*)processedServers;
-(bool)processedAllBackupServers:(NSMutableArray*)processedServers;
-(void)stopProcessDownloadSpeedThread;
-(void)stopThreads;
-(int)getAverageDownloadSpeed:(int)numberOfPoints;
-(void)detectConnectResultErrors;
-(void)resetConnections:(bool)AndStop;
-(void)resetConnections:(bool)AndStop InFreeMode:(bool)FreeMode;
-(void)checkBackupServerArticlesValidity;
-(void)addMonthlyTransferred:(int)bytes;
-(void)resetMonthlyTransferredLastWarningMonth;
-(void)detectMonthlyTransferredMonthChanged;
-(void)detectServerHandlerVolumeLimitReached;
-(void)processDownloadSpeed;
-(bool)isQueueComplete;
-(void)processSchedulerNotification;
-(bool)queueCompletedShowMessageCheck;
-(void)preventSleepWhenDownloading;
-(void)forceWriteAllNZBs;
-(void)restartNetworkAPIServer;

#pragma mark
#pragma mark Rest
-(void)dealloc;
-initWithNotifications:(NSObject<UINotificationsProtocol>*)UINotifications;

@end
