//
//  SearchProtocol.h
//  NZBVortex
//
//  Created by Ger Teunis on 24-12-11.
//  Copyright (c) 2011 Ger Teunis. All rights reserved.
//

#import <Foundation/Foundation.h>
@class UsenetSearchResult;

typedef enum {
	SearchSortTypeAge,
	SearchSortTypeSize
} SearchSortType;

typedef enum {
	SearchSortDirectionDescending,
	SearchSortDirectionAscending
} SearchSortDirection;


@protocol UsenetSearchProtocol <NSObject>

#pragma mark - Properties
-(NSString*)name;
-(NSArray*)allowedNumberOfResults;
-(NSArray*)searchResults;
-(bool)sortingAllowed;
-(bool)freeSizeRangeAllowed;
-(NSArray*)sizeRangeOptions;
-(bool)pagingAllowed;
-(bool)canCompileRSSURL;
-(int)currentPage;
-(int)numberOfPages;
-(int)totalNumberOfResult;
-(bool)setCurrentPage:(int)newCurrentPage;

#pragma mark - Categories
-(bool)categoriesSupported;
-(NSArray*)selectedCategoryIDs;
-(NSDictionary*)fetchCategoryDetails;
-(void)addSelectedCategoryId:(NSNumber*)category;
-(void)removeSelectedCategoryId:(NSNumber*)category;
-(BOOL)queryAllCategories;
-(void)setQueryAllCategories:(BOOL)queryAll;

#pragma mark - Search functions
-(NSString*)addQuery:(NSString*)query orKeywords:(NSArray*)orKeywords;
-(void)clearSearchResults;
-(bool)performSearchWithQuery:(NSString*)query
						  minSize:(NSNumber*)minSize
						  maxSize:(NSNumber*)maxSize
						   maxAge:(NSNumber*)maxAge
						 hideSpam:(bool)hideSpam
					 completeOnly:(bool)completeOnly
				  hideCrossPosted:(bool)hideCrossPostPosted
					 pageSize:(int)pageSize
						 sortType:(SearchSortType)sortType
					sortDirection:(SearchSortDirection)sortDirection;
-(bool)downloadSearchResult:(UsenetSearchResult*)result
             downloadFolder:(NSString*)downloadFolder
               downloadNext:(bool)downloadNext
                 mergerUUID:(NSString*)MergeAddUUID
                  addPaused:(bool)addPaused;

//minimal required methods
-(bool)performSearchWithURL:(NSString *)url;
-(NSString*)compileQueryURL:(NSString*)query
                    minSize:(NSNumber*)minSize
                    maxSize:(NSNumber*)maxSize
                     maxAge:(NSNumber*)maxAge
                   hideSpam:(bool)hideSpam
               completeOnly:(bool)completeOnly
            hideCrossPosted:(bool)hideCrossPostPosted
				   pageSize:(int)pageSize
                currentPage:(int)currentPage
                   sortType:(SearchSortType)sortType
              sortDirection:(SearchSortDirection)sortDirection;



@end
